"""Kubernetes class for Alert Exporter."""
import logging
import sys

from kubernetes import client, config


class Kubernetes:
    """
    A wrapper for the Kubernetes client
    """

    def __init__(self, context: str) -> None:
        config.load_kube_config()
        contexts, active_context = config.list_kube_config_contexts()
        available_contexts = [c["name"] for c in contexts]
        if context and context not in available_contexts:
            logging.error(
                f"Context {context} not found in {config.KUBE_CONFIG_DEFAULT_LOCATION}"
            )
            logging.info("Available contexts:")
            print("\n".join(available_contexts))
            sys.exit(1)

        if context:
            logging.info(f"Using context {context}")
            config.load_kube_config(context=context)
        else:
            logging.info(f"Using context {active_context.get('name')}")
        self.client = client.CoreV1Api()

    def get_prometheus_rules(self) -> None:
        self.rules = []
        crd_api = client.CustomObjectsApi()
        rules = crd_api.list_cluster_custom_object(
            group="monitoring.coreos.com", version="v1", plural="prometheusrules"
        )
        for pr in rules.get("items", []):
            prometheus_rule_name = pr["metadata"]["name"]
            logging.info(f"Prometheus Rule - {prometheus_rule_name}")
            for grp in pr["spec"]["groups"]:
                self.rules += [
                    {
                        "promRuleName": prometheus_rule_name,
                        "name": "/".join(
                            filter(None, [r.get("alert"), grp.get("name")])
                        ),
                        "description": r.get("annotations", {}).get("description", ""),
                        "summary": r.get("annotations", {}).get("summary", ""),
                        "expr": r.get("expr", ""),
                        "for": r.get("for", ""),
                        "severity": r.get("labels", {}).get("severity", ""),
                        "runbook": r.get("annotations", {}).get("runbook_url", ""),
                    }
                    for r in grp["rules"]
                ]



"""Cloudwatch class for Alert Exporter."""

import logging
import traceback
from datetime import timedelta
from time import sleep

import boto3
from botocore.exceptions import ClientError
from humanize import naturaldelta

COMPARISON_OPERATORS = {
    "GreaterThanOrEqualToThreshold": ">=",
    "GreaterThanThreshold": ">",
    "LessThanOrEqualToThreshold": "<=",
    "LessThanThreshold": "<",
}


class Cloudwatch:
    """
    A wrapper for the Cloudwatch boto client
    """

    def __init__(self, profile: str, region: str) -> None:
        self.session = boto3.session.Session(profile_name=profile)
        if region:
            self.regions = [region]
        else:
            self.regions = self.session.get_available_regions(service_name="cloudwatch")

    def init_client(self, region: str) -> None:
        self.client = self.session.client("cloudwatch", region_name=region)

    def get_alarms(self, debug: bool) -> None:
        self.rules = []
        for region in self.regions:
            self.init_client(region)
            try:
                alarms = self.client.describe_alarms()
            except ClientError as e:
                logging.warning(
                    f"Error while describing alarms in region {region}: {e}"
                )
                if debug:
                    print(traceback.format_exc())
                continue
            for alarm_type in ["CompositeAlarms", "MetricAlarms"]:
                self.rules += [
                    {
                        "region": region,
                        "type": alarm_type,
                        "name": r.get("AlarmName", ""),
                        "description": r.get("AlarmDescription", ""),
                        "rule": (
                            f'{r.get("MetricName")}'
                            f' {COMPARISON_OPERATORS[r.get("ComparisonOperator")]}'
                            f' {r.get("Threshold")}'
                            f' for {r.get("EvaluationPeriods")} datapoints within'
                            f' {naturaldelta(timedelta(seconds=r.get("Period")))}'
                        ),
                    }
                    for r in alarms[alarm_type]
                ]
            sleep(0.5)



"""Console script for alert_exporter."""
import logging
import os
import sys
from argparse import SUPPRESS, ArgumentParser, Namespace
from pathlib import Path

from jinja2 import (
    Environment,
    FileSystemLoader,
    PackageLoader,
    Template,
    select_autoescape,
)

from alert_exporter.sources.cloudwatch import Cloudwatch
from alert_exporter.sources.kubernetes import Kubernetes

AVAILABLE_FORMATS = {
    "csv": "csv",
    "html": "html",
    "md": "markdown",
    "yaml": "yaml",
    "yml": "yaml",
}


def init_args() -> Namespace:
    parser = ArgumentParser(
        description="Extract alerts configured in different sources"
        " (eg: Prometheus Rules, CloudWatch Alarms, etc.)",
        usage=SUPPRESS,
    )
    parser.add_argument(
        "--log-level",
        default="WARNING",
        choices=[
            "DEBUG",
            "INFO",
            "WARNING",
            "ERROR",
        ],
    )
    parser.add_argument("-o", "--output-file", required=True)
    parser.add_argument("--jinja-template", nargs="?")
    parser.add_argument("-f", "--format", choices=list(set(AVAILABLE_FORMATS.values())))
    parser.add_argument("--prometheus", default=False, action="store_true")
    parser.add_argument("--context", nargs="?")
    parser.add_argument("--cloudwatch", default=False, action="store_true")
    parser.add_argument("--aws-profile", default=os.getenv("AWS_PROFILE", None))
    parser.add_argument(
        "--aws-region",
        help="Specific region to target. Default: Iterate over all regions available.",
    )
    args = parser.parse_args()

    return args


def get_template_file(
    output_file: str, output_format: str, jinja2_template: str
) -> str:
    if output_format:
        if output_format == "markdown":
            return "alerts.md.jinja"
        else:
            return f"alerts.{output_format}.jinja"
    else:
        if jinja2_template:
            return jinja2_template
    file_extension = output_file.split(".")[-1]
    if not file_extension:
        logging.error(
            "The output format couldn't be automatically guessed from the"
            " file extension. Please use one of the available formats"
            " or provide a Jinja2 template."
        )
        sys.exit(1)
    if file_extension in AVAILABLE_FORMATS:
        if AVAILABLE_FORMATS[file_extension] == "markdown":
            return "alerts.md.jinja"
        return f"alerts.{AVAILABLE_FORMATS[file_extension]}.jinja"
    logging.error(
        f"No output format detected for the output file extension '{file_extension}'."
        " If you want to export in a custom format,"
        " you need to provide a Jinja2 file with the --jinja-template flag."
    )
    sys.exit(1)


def render_template(template: Template, rules: list, output_file: str) -> None:
    rendered = template.render(rules=rules)
    if "/" in output_file:
        absolute_file = output_file
    else:
        absolute_file = f"{os.getcwd()}/{output_file}"
    with open(absolute_file, "w+") as f:
        f.write(rendered)


def main():
    args = init_args()
    logging.basicConfig(
        level=args.log_level,
        format="%(asctime)s - %(levelname)s - %(message)s",
    )

    rules = []
    if args.prometheus:
        k = Kubernetes(context=args.context)
        k.get_prometheus_rules()
        rules += k.rules
    if args.cloudwatch:
        c = Cloudwatch(profile=args.aws_profile, region=args.aws_region)
        c.get_alarms(debug=bool(args.log_level == "DEBUG"))
        rules += c.rules
    if len(rules) == 0:
        logging.warning("No alert rule found.")
        return 1

    if args.jinja_template:
        path = Path(args.jinja_template)
        if not path.exists():
            logging.error(f"The template file {str(path)} does not exist.")
            return 1
        template_file = path.name
        env = Environment(loader=FileSystemLoader(path.parent.absolute()))
    else:
        template_file = get_template_file(
            output_file=args.output_file,
            output_format=args.format,
            jinja2_template=args.jinja_template,
        )
        env = Environment(
            loader=PackageLoader("alert_exporter"), autoescape=select_autoescape()
        )
    template = env.get_template(template_file)
    render_template(
        template=template,
        rules={
            "prometheus": k.rules if args.prometheus else [],
            "cloudwatch": c.rules if args.cloudwatch else [],
        },
        output_file=args.output_file,
    )
    return 0


if __name__ == "__main__":
    sys.exit(main())
